#include "Thermo.h"

#include "GenericTypeDefs.h"

BYTE tH, tL, Conf;

SHORT
ds1820_read (int channel)
{
  BYTE  busy = 0, temp1, temp2;
  SHORT temp3;

  onewire_reset (channel);
  onewire_write (channel, DS18_SKIP_ROM);
  onewire_write (channel, DS18_CONV_T);
  DelayMs (750); //DelayMs (200);
  
  while (busy == 0)
    busy = onewire_read (channel);

  onewire_reset (channel);
  onewire_write (channel, DS18_SKIP_ROM);
  onewire_write (channel, DS18_READ_SCRATCH);
  temp1 = onewire_read (channel);
  temp2 = onewire_read (channel);
  tH = onewire_read (channel);
  tL = onewire_read (channel);
  Conf = onewire_read (channel);

  temp3 = ( (((WORD)temp2) <<8) | ((WORD)temp1) );  

  DelayMs (200);
  return (temp3);
}

void
init_1wire (int channel)
{
  //for 10 bit resolution mod
  onewire_write (channel, DS18_SKIP_ROM);
  onewire_write (channel, DS18_WRITE_SCRATCH);  // Write scratchpad (Two bytes follow: T_H and T_L)

  //this should be done for proper working of DS18B20
  onewire_write (channel, DS18_2X_MAX_T);   // T_H register content
  onewire_write (channel, DS18_2X_MIN_T);   // T_L register content
  onewire_write (channel, 127);   // What?

  onewire_reset (channel);
  onewire_write (channel, DS18_SKIP_ROM);
  onewire_write (channel, DS18_COPY_SCRATCH);
  //delay_ms (15);
  DelayMs (15);
}

/*-------1-wire definitions-------*/
void set_tris(int channel, BOOL val)
{
	switch ( channel )
	{
		case 0:
			OW0_TRIS = val;
			break;
		case 1:
			OW1_TRIS = val;
			break;
		case 2:
			OW2_TRIS = val;
			break;
		case 3:
			OW3_TRIS = val;
			break;
		default:
			return;
	}
}

void set_bit(int channel, BOOL val)
{
	switch ( channel )
	{
		case 0:
			OW0_IO = val;
			break;
		case 1:
			OW1_IO = val;
			break;
		case 2:
			OW2_IO = val;
			break;
		case 3:
			OW3_IO = val;
			break;
		default:
			return;
	}
}

BOOL get_bit(int channel)
{
	switch ( channel )
	{
		case 0:
			return OW0_IO;
		case 1:
			return OW1_IO;
		case 2:
			return OW2_IO;
		case 3:
			return OW3_IO;
		default:
			return 0;
	}
}
/*******************1-wire communication functions********************/


void
onewire_reset (int channel)                // OK if just using a single permanently connected device
{

  //ONE_WIRE_TRIS = 0;
	set_tris(channel, 0);
  //ONE_WIRE_IO = 0;
	set_bit(channel, 0);
  delay_us (500);               // pull 1-wire low for reset pulse
  //ONE_WIRE_TRIS = 1;
	set_tris(channel, 1);
  delay_us (500);               // wait-out remaining initialisation window.
}

/*********************** onewire_write() ********************************/
/*This function writes a byte to the sensor.*/
/* */
/*Parameters: byte - the byte to be written to the 1-wire */
/*Returns: */
/*********************************************************************/

void
onewire_write (int channel, int data)
{
  int count;
  BOOL b;

  for (count = 0; count < 8; ++count)
    {
      //ONE_WIRE_TRIS = 0;
	set_tris(channel, 0);
      //ONE_WIRE_IO = 0;
	set_bit(channel, 0);
      delay_us (2);             // pull 1-wire low to initiate write time-slot.

      //ONE_WIRE_IO = (data & 0x01); data >>= 1;    // set output bit on 1-wire
	b = (data & 0x01) ? 1 : 0;
	set_bit(channel, b);
	data>>=1;

      delay_us (60);            // wait until end of write slot.
      //ONE_WIRE_TRIS = 1;
	set_tris(channel, 1);
      delay_us (2);             // for more than 1us minimum.
    }
}

/*********************** read1wire() *********************************/
/*This function reads the 8 -bit data via the 1-wire sensor. */
/* */
/*Parameters: */
/*Returns: 8-bit (1-byte) data from sensor */
/*********************************************************************/

int
onewire_read (int channel)
{
  int count, data = 0;

  for (count = 0; count < 8; ++count)
    {
      //ONE_WIRE_TRIS = 0;
	set_tris(channel, 0);
      //ONE_WIRE_IO = 0;
	set_bit(channel, 0);
      delay_us (2);             // pull 1-wire low to initiate read time-slot.
      //ONE_WIRE_TRIS = 1;
	set_tris(channel, 1);
      delay_us (8);             // let device state stabilise,
      //data >>= 1; if (ONE_WIRE_IO) { data |= 0x80; }
    data >>= 1; 
	if ( get_bit(channel) ) data |= 0x80;

      delay_us (120);           // wait until end of read slot.
    }

  return (data);
}
