/*********************************************************************
 *
 *  Code entry point for tempdude temperature logger firmware.
 *
 *********************************************************************
 * Processor:       PIC18 on Olimex PICMINIWEB
 * Complier:        Microchip C18 v3.02 or higher


 * This file contains primarily code written by
 * Jon Langseth, and is made available under a
 * BSD style license.
 * 
 * However, a majority of the code is inherited from
 * the Microchip TCP/IP stack version 4.02.
 * That code is licensed under limited bsd style.
 * Both license texts follow.

Copyright (c) 2010, LiNC/Jon Langseth
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by LiNC/Jon Langseth.
4. Neither the name of LiNC, Jon Langseth nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY LiNC ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL LiNC BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright  2002-2007 Microchip Technology Inc.  All rights 
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and 
 * distribute: 
 * (i)  the Software when embedded on a Microchip microcontroller or 
 *      digital signal controller product (Device) which is 
 *      integrated into Licensees product; or
 * (ii) ONLY the Software driver source files ENC28J60.c and 
 *      ENC28J60.h ported to a non-Microchip device used in 
 *      conjunction with a Microchip ethernet controller for the 
 *      sole purpose of interfacing with the ethernet controller. 
 *
 * You should refer to the license agreement accompanying this 
 * Software for additional information regarding your rights and 
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT 
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT 
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL 
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR 
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF 
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS 
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE 
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER 
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT 
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
/*
 * Following define uniquely deines this file as main
 * entry/application In whole project, there should only be one such
 * definition and application file must define AppConfig variable as
 * described below.
 */
#define THIS_IS_STACK_APPLICATION

#include "TCPIP Stack/TCPIP.h"
#include "Thermo.h"

// This is used by other stack elements.
// Main application must define this and initialize it with proper values.
APP_CONFIG AppConfig;
BYTE AN0String[8];
BYTE myDHCPBindCount = 0xFF;

#if !defined(STACK_USE_DHCP_CLIENT)
	#define DHCPBindCount	(0xFF)
#endif

#pragma config XINST=OFF, WDTEN=OFF, FOSC=HSPLL


// Private helper functions.
// These may or may not be present in all applications.
static void InitAppConfig(void);
static void InitializeBoard(void);
static void ProcessIO(void);
static void FormatNetBIOSName(BYTE Name[16]);
#if defined(MPFS_USE_EEPROM) && defined(STACK_USE_MPFS)
	static void SaveAppConfig(void);
#else
	#define SaveAppConfig()
#endif


#define VERSION 		"v4.02"		// TCP/IP stack version
#define BAUD_RATE       (19200)		// bps

#define STMP_DEF_PORT		13579
#define STMP_CFG_PORT		13580
#define STMP_SEND_INTERVAL	(TICK_SECOND*30)
#define STMP_ARP_TIMEOUT	(TICK_SECOND*5)

static NODE_INFO	STMPServer;
static int			STMPServerPort = STMP_DEF_PORT;

static int sensors[4];

void StmpInit (void);
void StmpTask (void);
void ConfigTask(void);

// NOTE: Several PICs, including the PIC18F4620 revision A3 have a RETFIE FAST/MOVFF bug
// The interruptlow keyword is used to work around the bug when using C18
extern void MACISR(void);

#pragma interruptlow LowISR

void LowISR(void)
{
    TickUpdate();
}

#pragma code lowVector=0x18
void LowVector(void){_asm goto LowISR _endasm}
#pragma code // Return to default code section

// Main application entry point.
void main(void)
{
    static TICK t = 0;

    // Initialize any application specific hardware.
    InitializeBoard();

    // Initialize all stack related components.
    // Following steps must be performed for all applications using
    // the Microchip TCP/IP Stack.
    TickInit();

#if defined(STACK_USE_MPFS)
    // Initialize Microchip File System module
    MPFSInit();
#endif

    // Initialize Stack and application related NV variables into AppConfig.
    InitAppConfig();
	// Initialize core stack layers (MAC, ARP, TCP, UDP)
    StackInit();

	StmpInit();

#if defined(STACK_USE_DHCP_CLIENT) || defined(STACK_USE_IP_GLEANING)
    if(!AppConfig.Flags.bIsDHCPEnabled)
    {
        // Force IP address display update.
        myDHCPBindCount = 1;
#if defined(STACK_USE_DHCP_CLIENT)
        DHCPDisable();
#endif
    }
#endif

    // Once all items are initialized, go into infinite loop and let
    // stack items execute their tasks.
    // If application needs to perform its own task, it should be
    // done at the end of while loop.
    // Note that this is a "co-operative mult-tasking" mechanism
    // where every task performs its tasks (whether all in one shot
    // or part of it) and returns so that other tasks can do their
    // job.
    // If a task needs very long time to do its job, it must be broken
    // down into smaller pieces so that other tasks can have CPU time.
    while(1)
    {
        // This task performs normal stack task including checking
        // for incoming packet, type of packet and calling
        // appropriate stack entity to process it.
        StackTask();

        ProcessIO();
		StmpTask();
		ConfigTask();

        // For DHCP information, display how many times we have renewed the IP
        // configuration since last reset.
        if(DHCPBindCount != myDHCPBindCount)
        {
            myDHCPBindCount = DHCPBindCount;

			#if defined(STACK_USE_ANNOUNCE)
				AnnounceIP();
			#endif
        }

    }
}

//----------------------------------------------------------------
void ConfigTask ( void )
{
	BYTE t;
	BYTE i;
	int cookie;
	static BYTE remote_addr[4];
	static int remote_port;
	static UDP_SOCKET	MySocket = INVALID_UDP_SOCKET;
	static TICK StartTime = 0;

	static enum
	{
		CFG_IDLE = 0,
		CFG_START,
		CFG_OPEN_SOCKET,
		CFG_SEND,
		CFG_RECV,
		CFG_DONE
	} state = CFG_IDLE;

	if ( state == CFG_IDLE )
	{
		if( BUTTON0_IO == 1 )
		{
			LED0_IO = 0;
			return;
		}
		else 
		{
			LED0_IO = 1;
			state++;
		}
	}
	switch(state)
	{
		case CFG_START:
			StartTime = TickGet();
			state++;
			break;

		case CFG_OPEN_SOCKET:
			MySocket = UDPOpen(STMP_CFG_PORT, NULL, STMP_CFG_PORT);
			if(MySocket == INVALID_UDP_SOCKET)
				break;
			state++;

		case CFG_SEND:
			if(!UDPIsPutReady(MySocket))
				break;

			UDPPut( 0xFE );
			UDPPut( 0x80 );
			UDPPut( 0x01 );
			// Send the packet
			UDPFlush();
			state++;
			break;

		case CFG_RECV:
		    if(TickGet() - StartTime >= 30*TICK_SECOND)
			{
				state++;
				break;
			}

			if(!UDPIsGetReady(MySocket))
				break;

			if ( ! UDPGet(&t) ) break;
			cookie = ((int)t)<<8u;
			if ( ! UDPGet(&t) ) break;
			cookie |=  t;

			// Continuing if we have recieved magic cookie.
			if ( cookie == 0x4040 )
			{
				// Read the next four bytes of the package (IP address)
				i = 0;
				while ( i < 4 )
				{
					UDPGet(&t);
					remote_addr[i] = t;
					i++;
				}

				// Recieve the port number as two bytes,
				// and push the recieved bytes in the TXbuffer immediately.
				UDPGet(&t);
				remote_port = ((int)t)<<8u;
				UDPGet(&t);
				remote_port |= t;

				// Two bytes magick cookie, plus six bytes config, 
				// Next by6te should be a terminating 0xFF
				if ( ! UDPGet(&t) ) break;
				if ( t != 0x3B ) break;

				// All is well. Send an ACK.
				UDPPut( 0x40 );
				UDPPut( 0x40 );
				UDPPut( 0x06 );
				UDPFlush();
				LED0_IO = 0;
				state++;
			}
			break;

		case CFG_DONE:
			UDPPut( 0x20 );
			UDPPut( 0x20 );
			UDPPutArray( remote_addr, 4 );
			UDPPut( 0x20 );
			UDPPut( remote_port>>8 );
			UDPPut( remote_port );
			UDPPut( 0x20 );
			UDPPut( 0x06 );
			UDPFlush();

			// Close the socket so it can be used by other modules
			UDPClose(MySocket);

			STMPServer.IPAddr.Val  = remote_addr[3];
			STMPServer.IPAddr.Val = STMPServer.IPAddr.Val<<8;
			STMPServer.IPAddr.Val |= remote_addr[2];
			STMPServer.IPAddr.Val = STMPServer.IPAddr.Val<<8;
			STMPServer.IPAddr.Val |= remote_addr[1];
			STMPServer.IPAddr.Val = STMPServer.IPAddr.Val<<8;
			STMPServer.IPAddr.Val |= remote_addr[0];

			STMPServerPort = remote_port; 

			// Save config to EEPROM here..
			SaveAppConfig();

			StartTime = 0;
			state = 0;
			return;
	}
}

void StmpInit ( void )
{

}

//------------------------------------------------------------------------------
void StmpTask ( void )
{
	static UDP_SOCKET	MySocket = INVALID_UDP_SOCKET;
	static TICK StartTime = 0;
	BYTE t;
	BYTE i;
	BYTE j;
	static enum
	{
		STMP_START = 0,
		STMP_RESOLVE_ARP,
		STMP_OPEN_SOCKET,
		STMP_SEND,
		STMP_DONE
	} state = STMP_DONE;

	if( (TickGet() - StartTime <= STMP_SEND_INTERVAL) && ( state == STMP_START ) ) return;

	switch(state)
	{
		case STMP_START:
			if ( STMPServer.IPAddr.Val != 0 )
				ARPResolve(&STMPServer.IPAddr);
			else
				state++;

			StartTime = TickGet();
			state++;
			break;

		case STMP_RESOLVE_ARP:
			if(!ARPIsResolved(&STMPServer.IPAddr, &STMPServer.MACAddr))
			{
				if(TickGet() - StartTime >= STMP_ARP_TIMEOUT )
				{
					state--;
				}
				break;
			}
			state++;

		case STMP_OPEN_SOCKET:
			if ( STMPServer.IPAddr.Val != 0 )
				MySocket = UDPOpen(STMP_DEF_PORT, &STMPServer, STMPServerPort);
			else
				MySocket = UDPOpen(STMP_DEF_PORT, NULL, STMP_DEF_PORT);

			if(MySocket == INVALID_UDP_SOCKET)
				break;

			state++;

		case STMP_SEND:
			if(!UDPIsPutReady(MySocket))
				break;

			j = 0;
			while ( j < 4 )
			{

				UDPPut('L');
				UDPPut('E');
				UDPPut('M');
				UDPPut('P');
				UDPPut('1');
				UDPPut( 0 );
				UDPPut( 0x01 );	// Type: NodeID
				UDPPut( 5u );   // Length: 5 (mfg + 3 bytes oid + sensorid)
				UDPPut( 0x04 ); // MFG ID 4...
				i = 3;
				while(i < 6u)
				{
					UDPPut(AppConfig.MyMACAddr.v[i]);
				    i++;
				}
				UDPPut( j+1 ); // Sensor #
	
				UDPPut( 0x08 ); // Type: Thermovalue
				UDPPut( 0x02 ); // Length: 2 bytes


				t= (BYTE) (sensors[j] >> 8ul );
				UDPPut(t);	
				t = (BYTE) sensors[j];
				UDPPut(t);
				UDPFlush();

				j++;
			}
				
			// Send the packet
			state++;
			break;

		case STMP_DONE:
			// Close the socket so it can be used by other modules
			UDPClose(MySocket);
			state = 0;
	    	StartTime = TickGet();
			return;
	}
}

static void ProcessIO(void)
{
	static TICK lastRun = 0; 
	BYTE i;

    if(TickGet() - lastRun >= 10*TICK_SECOND)
    {
            lastRun = TickGet();
			i = 0;
			while ( i < 4 )
			{
				sensors[i] = ds1820_read(i);
				if ( sensors[i] == 0xFFFF )
					sensors[i] = ds1820_read(i);

				i++;
			}
    }
}

/*********************************************************************
 * Function:        void InitializeBoard(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Initialize board specific hardware.
 *
 * Note:            None
 ********************************************************************/
static void InitializeBoard(void)
{
	// LEDs
	LED0_TRIS = 0;
	LED0_IO = 0;

	// Ivan
	TRISC &= 0x3C;
	TRISB &= 0xFD;
	TRISA &= 0xD0;

	// Enable 4x/5x PLL on PIC18F87J10, PIC18F97J60, etc.
    OSCTUNE = 0x40;

	// Set up analog features of PORTA

	ADCON0 = 0x01;		// ADON, Channel 0
	ADCON1 = 0x0E;		// Vdd/Vss is +/-REF, AN0 is analog
	TRISA = 0x23;
	ADCON2 = 0xBE;			// Right justify, 20TAD ACQ time, Fosc/64 (~21.0kHz)

    // Enable internal PORTB pull-ups
    INTCON2bits.RBPU = 0;

	// Configure USART
    TXSTA = 0x20;
    RCSTA = 0x90;

	// See if we can use the high baud rate setting
#if ((INSTR_FREQ+2*BAUD_RATE)/BAUD_RATE/4 - 1) <= 255
	SPBRG = (INSTR_FREQ+2*BAUD_RATE)/BAUD_RATE/4 - 1;
	TXSTAbits.BRGH = 1;
#else	// Use the low baud rate setting
	SPBRG = (INSTR_FREQ+8*BAUD_RATE)/BAUD_RATE/16 - 1;
#endif


	// Enable Interrupts
	RCONbits.IPEN = 1;		// Enable interrupt priorities
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;


}

/*********************************************************************
 * Function:        void InitAppConfig(void)
 *
 * PreCondition:    MPFSInit() is already called.
 *
 * Input:           None
 *
 * Output:          Write/Read non-volatile config variables.
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            None
 ********************************************************************/
static void InitAppConfig(void)
{
    BYTE c;
    BYTE *p;

	AppConfig.Flags.bIsDHCPEnabled = TRUE;
	AppConfig.Flags.bInConfigMode = TRUE;
	AppConfig.MyMACAddr.v[0] = MY_DEFAULT_MAC_BYTE1;
	AppConfig.MyMACAddr.v[1] = MY_DEFAULT_MAC_BYTE2;
	AppConfig.MyMACAddr.v[2] = MY_DEFAULT_MAC_BYTE3;
	AppConfig.MyMACAddr.v[3] = MY_DEFAULT_MAC_BYTE4;
	AppConfig.MyMACAddr.v[4] = MY_DEFAULT_MAC_BYTE5;
	AppConfig.MyMACAddr.v[5] = MY_DEFAULT_MAC_BYTE6;
	AppConfig.MyIPAddr.Val = MY_DEFAULT_IP_ADDR_BYTE1 | MY_DEFAULT_IP_ADDR_BYTE2<<8ul | MY_DEFAULT_IP_ADDR_BYTE3<<16ul | MY_DEFAULT_IP_ADDR_BYTE4<<24ul;
	AppConfig.DefaultIPAddr.Val = AppConfig.MyIPAddr.Val;
	AppConfig.MyMask.Val = MY_DEFAULT_MASK_BYTE1 | MY_DEFAULT_MASK_BYTE2<<8ul | MY_DEFAULT_MASK_BYTE3<<16ul | MY_DEFAULT_MASK_BYTE4<<24ul;
	AppConfig.DefaultMask.Val = AppConfig.MyMask.Val;
	AppConfig.MyGateway.Val = MY_DEFAULT_GATE_BYTE1 | MY_DEFAULT_GATE_BYTE2<<8ul | MY_DEFAULT_GATE_BYTE3<<16ul | MY_DEFAULT_GATE_BYTE4<<24ul;
	AppConfig.PrimaryDNSServer.Val = MY_DEFAULT_PRIMARY_DNS_BYTE1 | MY_DEFAULT_PRIMARY_DNS_BYTE2<<8ul  | MY_DEFAULT_PRIMARY_DNS_BYTE3<<16ul  | MY_DEFAULT_PRIMARY_DNS_BYTE4<<24ul;
	AppConfig.SecondaryDNSServer.Val = MY_DEFAULT_SECONDARY_DNS_BYTE1 | MY_DEFAULT_SECONDARY_DNS_BYTE2<<8ul  | MY_DEFAULT_SECONDARY_DNS_BYTE3<<16ul  | MY_DEFAULT_SECONDARY_DNS_BYTE4<<24ul;

	// Load the default NetBIOS Host Name
	memcpypgm2ram(AppConfig.NetBIOSName, (ROM void*)MY_DEFAULT_HOST_NAME, 16);
	FormatNetBIOSName(AppConfig.NetBIOSName);

    XEEBeginRead(0x0000);
    c = XEERead();
    XEEEndRead();

    // When a record is saved, first byte is written as 0x57 to indicate
    // that a valid record was saved.  Note that older stack versions 
	// used 0x55.  This change has been made to so old EEPROM contents 
	// will get overwritten.  The AppConfig() structure has been changed,
	// resulting in parameter misalignment if still using old EEPROM 
	// contents.
    if(c == 0x61u)
    {
        LED0_IO ^= 1;
        XEEBeginRead(0x0001);

        p = (BYTE*)&AppConfig;
        for ( c = 0; c < sizeof(AppConfig); c++ )
            *p++ = XEERead();

        p = (BYTE*)&STMPServer;
		for ( c = 0; c< sizeof(STMPServer); c++ )
            *p++ = XEERead();

        p = (BYTE*)&STMPServerPort;
		for ( c = 0; c< sizeof(STMPServerPort); c++ )
            *p++ = XEERead();

        XEEEndRead();
        LED0_IO ^= 1;
    }
    else
        SaveAppConfig();
}

static void SaveAppConfig(void)
{
    BYTE c;
    BYTE *p;
    //BYTE *s;


    XEEBeginWrite(0x0000);
    XEEWrite(0x61);

    p = (BYTE*)&AppConfig;
    for ( c = 0; c < sizeof(AppConfig); c++ )
    {
        LED0_IO ^= 1;
        XEEWrite(*p++);
    }
    p = (BYTE*)&STMPServer;
    for ( c = 0; c < sizeof(STMPServer); c++ )
    {
        XEEWrite(*p++);
    }
    p = (BYTE*)&STMPServerPort;
    for ( c = 0; c < sizeof(STMPServerPort); c++ )
    {
        XEEWrite(*p++);
    }

    XEEEndWrite();
}


// NOTE: Name[] must be at least 16 characters long.
// It should be exactly 16 characters, as defined by NetBIOS spec.
static void FormatNetBIOSName(BYTE Name[])
{
	BYTE i;

	Name[15] = '\0';
	strupr((char*)Name);
	i = 0;
	while(i < 15u)
	{
		if(Name[i] == '\0')
		{
			while(i < 15u)
			{
				Name[i++] = ' ';
			}
			break;
		}
		i++;
	}
}

